KEY_FOCAL_SEQS = "focal_seqs"
KEY_BACKGROUND_SEQS = "background_seqs"
KEY_OUTDIR = "outdir"
KEY_REFERENCE = "reference"
KEY_MASTER_FASTA = "master_fasta"
KEY_THREADS = "nthreads"
KEY_SNPS_ONLY = "snps_only"
KEY_RENAME = "rename"
KEY_NAMES_CSV = "names_csv"
KEY_PREFIX = "prefix"
KEY_CONST_SITES = "const_sites"
KEY_FILTER = "filter"
KEY_GENOME_COMPLETENESS = "genome_completeness"
KEY_GENOME_LENGTH = "genome_length"
KEY_REPORT = "report"